import java.util.Calendar;
import java.util.GregorianCalendar;


public class EditDistance {

    public static int recursion_steps = 0;

    public static void main(String[] args) {
        if (args.length == 2) {
            String x = (String) args[0];
            String y = (String) args[1];
            computeEditDistanceRecursive(x, y);
            computeEditDistanceIterative(x, y);
        }
    }

    // Berechnet die Edit-Distanz rekursiv
    public static void computeEditDistanceRecursive(String x, String y) {
        // mit Hilfe des Kalenders wird die Anfangszeit genommen
        Calendar calender = new GregorianCalendar();
        long start = calender.getTimeInMillis();
        // die Distanz wird rekursiv berechnet
        int distance_recursiv = recursionEditDistance(x, y, x.length(),
                y.length());
        // die Zeit nach Durchlauf der Berechnung wird genommen
        calender = new GregorianCalendar();
        long stop = calender.getTimeInMillis();
        // die Differenz aus Start- und Stopp-Zeit ist die Laufzeit der
        // Berechnung
        long durance = stop - start;
        System.out.println("Edit-Distanz rekursiv berechnet: "
                + distance_recursiv);
        System.out.println("Anzahl Rekursionsschritte: " + recursion_steps);
        System.out.println("Zeit in Millisekunden bei der Rekursion: "
                + durance);
        System.out.println();
    }

    // Rekursionsschritt fuer die Berechnung der Edit-Distanz
    public static int recursionEditDistance(String x, String y, int i, int j) {
        recursion_steps++;
        // Basisfall 1
        if (i == 0) {
            return j;
        }
        // Basisfall 2
        if (j == 0) {
            return i;
        }
        // Berechnungen, um einen weiteren Rekursionsschritt mit neuen Werten
        // durchlaufen zu koennen
        int copy_sub = recursionEditDistance(x, y, i - 1, j - 1);
        if (x.charAt(i - 1) != y.charAt(j - 1)) {
            copy_sub++;
        }
        int insert = recursionEditDistance(x, y, i - 1, j) + 1;
        int delete = recursionEditDistance(x, y, i, j - 1) + 1;
        return returnMinimum(copy_sub, insert, delete);
    }

    // Edit-Distanz wird iterativ berechnet
    public static int computeEditDistanceIterative(String x, String y) {
        // mit Hilfe des Kalenders wird die Anfangszeit genommen
        Calendar calender = new GregorianCalendar();
        long start = calender.getTimeInMillis();
        int[][] editMatrix = new int[x.length() + 1][y.length() + 1];
        //Initialisierung vertikal
        for (int i = 0; i < editMatrix.length; i++) {
            editMatrix[i][0] = i;
        }
        //Initialisierung horizontal
        for (int i = 0; i < editMatrix[0].length; i++) {
            editMatrix[0][i] = i;
        }
        //Berechnung der anderen Eintraege
        for (int i = 1; i < editMatrix.length; i++) {
            for (int j = 1; j < editMatrix[i].length; j++) {
                int copy_sub = editMatrix[i - 1][j - 1];
                if (x.charAt(i - 1) != y.charAt(j - 1)) {
                    copy_sub++;
                }
                int insert = editMatrix[i - 1][j] + 1;
                int delete = editMatrix[i][j - 1] + 1;
                editMatrix[i][j] = returnMinimum(copy_sub, insert, delete);
            }
        }
        // die Zeit nach Durchlauf der Berechnung wird genommen
        calender = new GregorianCalendar();
        long stop = calender.getTimeInMillis();
        // die Differenz aus Start- und Stopp-Zeit ist die Laufzeit der
        // Berechnung
        long durance = stop - start;
        System.out.println("Zeit in Millisekunden bei iterativer Berechnung: "
                + durance);

        // Ausgabe der Matrix:
        System.out.println("Die Edit-Matrix: ");
        System.out.print("  - ");
        for (int i = 0; i < y.length(); i++) {
            System.out.print(y.charAt(i) + " ");
        }
        System.out.println();
        for (int i = 0; i < editMatrix.length; i++) {
            if (i == 0) {
                System.out.print("- ");
            } else {
                System.out.print(x.charAt(i - 1) + " ");
            }
            for (int j = 0; j < editMatrix[i].length; j++) {
                System.out.print(editMatrix[i][j] + " ");
            }
            System.out.println();
        }
        System.out.println("Edit-Distanz: " +
                editMatrix[x.length()][y.length()]);
        return editMatrix[x.length()][y.length()];

    }

    // Berechnet das Minimum dreier Zahlen
    public static int returnMinimum(int a, int b, int c) {
        if (a <= b && a <= c) {
            return a;
        }
        if (b <= a && b <= c) {
            return b;
        }
        return c;
    }
}
 
